TUTORIAL DE SUPERCOLLIDER 3 - PARTE 7
Rodrigo F. Cadiz, basado en apuntes de Gary S. Kendall
Northwestern University

(
Server.internal.boot;
Server.default = Server.internal;
s = Server.default;
)


Amplitude Modulation

Type #1 Ring/Balanced/Double-Sideband-Suppressed-Carrier Modulation

The amplitude of one signal varies according to another signal.

Slow amplitude modulation, that is, when the modulating frequency is less than 20 Hz., can
sound like a steady rhythm.  But when the modulating frequency is above 20 Hz, it sounds 
like a continuous tone with sidebands.

{SinOsc.ar(1000,0) * SinOsc.ar(Line.kr(1,50,30),0) }.play; 
// Nothing more than two sinusoids multiplied


Slow modulation of the modulator frequency reveals the sound of sidebands moving up and down simultaneously.

{
	var modFreq;
	modFreq = Line.kr(0, 600, 30); // Modulation frequency goes from 0 to 600 Hz
	SinOsc.ar(1000,0) * SinOsc.ar(modFreq,0) 
}.play;

The multiplication of one signal by another is better done with the mul: argument.

{
	var modFreq, amp;
	modFreq = Line.kr(0, 600, 30);
	amp = SinOsc.ar(modFreq,0);
	SinOsc.ar(1000,0,amp); // Rewritten for faster multiply
}.play;



Type #2 AM/Double-Sideband Modulation

Here the modulation index provides a way of controlling the amount of modulation
and simultaneously the amount of energy in the sidebands.

This example is easy to see in the time domain.

{
	var modIndex, modSignal;
	modIndex = Line.kr(0,1,12);		// Modulation Index goes from 0 to 1
	// Normalization is needed if you want to keep the signal level constant
	modSignal = (1 + (modIndex * SinOsc.ar(378,0))) / (1 + modIndex);
	modSignal * SinOsc.ar(1000,0,0.5);
}.play;

Below we use a conditional to mimic the behavior of an analog system. Overmodulation occures 
when the modIndex is greater than 1.  Here like in an analog system, the modution signal 
can not become negative and therefore the process in non-linear and produces many sidebands.

{
	var modIndex, modSignal;
	modIndex = Line.kr(0,5,10);
	modSignal = (1 + (modIndex * SinOsc.ar(79,0))) / (1 + modIndex);
	modSignal = modSignal.max(0); // passes just the positive part of the control signal
	modSignal * SinOsc.ar(1000,0,0.5);
}.play;



Frequency Modulation

In the following example the frequency of the audio rate oscillator is modulated by the control rate oscillator.

Carrier frequency is 700 Hz and the frequency deviation is 300 Hz.  The control frequency is only 0.5 Hz.

{
	SinOsc.ar(		// carrier oscillator
		SinOsc.ar(	// modulator oscillator
			0.5,		// modulating frequency
			0,		// zero phase
			300,		// mul field is the freqency deviation
			700		// add field is the carrier frequency
					// These settings of mul and add will cause the
					// frequency to vary from -300+700 to +300+700 Hz
					// which equals from 400 to 1000 Hz
		),
		0,			// zero phase
		0.5			// amplitude 0.5
	)
}.play;


The control frequency will now increate linearly from 0.5 to 100 Hz.  Note the changes in the resulting sound from a sine wave with changing pitch to a complex tone with multiple sidebands.

// sweep freq of LFO
{
	SinOsc.ar(			// carrier oscillator
		SinOsc.ar(		// modulator oscillator
			XLine.kr(		// make an exponential line generator for modulating freuqency
				1,		// begin at 1 Hz
				1000,	// end at 1000 Hz
				20		// in 20 seconds
			),
			0,			// zero phase
			300,		// mul field is the freqency deviation
			1000		// add field is the carrier frequency
		),
		0,				// zero phase
		0.5				// amplitude 0.5
	)
}.play;

By changing the frequency deviation (and thus, the modulation index) the distribution of energy in the sidebands changes.

(The nested style used above can become hard to read, so often it is preferable to use variables to make it  more readable.)

This one makes it easy to see the frequency modulation in the time domain.

{
	var freqDev, freq;		// declare the variables we will use in this function.
	
	freqDev = Line.kr(		// make a linear line generator
				({
	var fundFreq, peakFMI, carrierFreq, modulatorFreq, env1, env2, amp, instFreqDv, instFreq;
	fundFreq = 440.0;	// fundamental frequency
	c = 3; m = 1;	// All harmonics
	peakFMI = 2;
	carrierFreq = c * fundFreq;		// C:M ratio determines the carrier and
	modulatorFreq = m * fundFreq;	//  the modulator frequencies
	
	env1 = Env.new([0,5.0,3.75,3.0,0]/5,[0.15,0.15,0.15,0.15],'linear');
	amp = EnvGen.kr(env1);
	//env2 = Env.new([3*peakFMI, peakFMI, 0.7 * peakFMI],[0.3,0.45],'linear');
	env2 = Env.new([0.3*peakFMI, peakFMI, 2.0 * peakFMI],[0.3,0.45],'linear');
	instFreqDv = modulatorFreq * EnvGen.kr(env2);
	
	instFreq = SinOsc.ar(modulatorFreq,0,instFreqDv,carrierFreq);
	SinOsc.ar(instFreq,0,amp);	
}.play;)
0,		// begin at 0 Hz
				600,		// end at 600 Hz
				20		// in 20 seconds
			);
			
	freq = SinOsc.ar(	// modulating signal
			100,		// modulating frequency fixed at 100 Hz
			0,		// zero phase
			freqDev,	// mul field is the frequency deviation
			1000		// add field is the carrier frequency
		);
		
	SinOsc.ar(		// create a sine oscillator
		freq, 		// frequency modulator
		0,			// zero phase
		0.5			// amplitude 0.5
	)
}.play;


The example below implements a very simple Chowning FM instrument in which 
	the carrier and the modulating frequency are identical and one envelope controls the amplitude and the
	modulation index.

{
		var env, amp, instFreqDv, instFreq;	
		//env = Env.linen(3.0, 3.0, 3.0, 0.5); 
		env = Env.linen(0.1, 0.2, 0.3, 0.5); 
		amp = EnvGen.kr(env);	// amplitude envelope
		
		// instantaneous frequency = carrierFreq + FreqDv * Sin(modulatorFreq)
		// carrierFreq = modulatorFreq = 500 Hz
		// FreqDv = 2000 Hz
		instFreqDv = amp * 2000.0;  // amp envelop is used to change 
				// instantaneous frequency deviation
		instFreq = SinOsc.ar(500,0,instFreqDv,500);  // modulator = 500 Hz
		SinOsc.ar(instFreq,0,amp);					// carrier = 500 Hz
}.play;

Here is an example in which the C:M ratios and peakFMI can be altered.

({
	var fundFreq, peakFMI, carrierFreq, modulatorFreq, env, amp, instFreqDv, instFreq;
	fundFreq = 400.0;	// fundamental frequency
	c = 1; m = 1;	// All harmonics
	//c = 1; m = 2;	// Missing every 2nd harmonic
	//c = 1; m = 3;	// Missing every 3rd harmonic
	//c = 5; m = 1;	// All harmonics and starts on 5th harmonic
	//c = 1; m = 1.414;  //  Inharmonic 
	peakFMI = 10;
	carrierFreq = c * fundFreq;		// C:M ratio determines the carrier and
	modulatorFreq = m * fundFreq;	//  the modulator frequencies
	
	env = Env.new([0,5.0,3.75,3.0,0]/5,[0.15,0.15,0.15,0.15],'linear');
	amp = EnvGen.kr(env);
	
	instFreqDv = peakFMI * amp * modulatorFreq;
	instFreq = SinOsc.ar(modulatorFreq,0,instFreqDv,carrierFreq);
	SinOsc.ar(instFreq,0,amp);	
}.play;)

Here is an example with separate envelopes for the amplitude and for the frequency modulation index.

({
	var fundFreq, peakFMI, carrierFreq, modulatorFreq, env1, env2, amp, instFreqDv, instFreq;
	fundFreq = 440.0;	// fundamental frequency
	c = 3; m = 1;	// All harmonics
	peakFMI = 2;
	carrierFreq = c * fundFreq;		// C:M ratio determines the carrier and
	modulatorFreq = m * fundFreq;	//  the modulator frequencies
	
	env1 = Env.new([0,5.0,3.75,3.0,0]/5,[0.15,0.15,0.15,0.15],'linear');
	amp = EnvGen.kr(env1);
	//env2 = Env.new([3*peakFMI, peakFMI, 0.7 * peakFMI],[0.3,0.45],'linear');
	env2 = Env.new([0.3*peakFMI, peakFMI, 2.0 * peakFMI],[0.3,0.45],'linear');
	instFreqDv = modulatorFreq * EnvGen.kr(env2);
	
	instFreq = SinOsc.ar(modulatorFreq,0,instFreqDv,carrierFreq);
	SinOsc.ar(instFreq,0,amp);	
}.play;)







